#!/bin/bash
set -e

source charms.reactive.sh

CUDA_VERSION=$(config-get cuda-version | awk 'BEGIN{FS="-"}{print $1}')
CUDA_SUB_VERSION=$(config-get cuda-version | awk 'BEGIN{FS="-"}{print $2}')
LAYER_BLACKLIST_CONF="/etc/modprobe.d/blacklist-layer-nvidia-cuda.conf"
LAYER_LD_CONF="/etc/ld.so.conf.d/layer-nvidia-cuda.conf"
LAYER_PROFILE_CONF="/etc/profile.d/layer-nvidia-cuda.sh"
LAYER_RC_CONF="/home/ubuntu/.bashrc"
ROOT_URL="http://developer.download.nvidia.com/compute/cuda/repos"
SUPPORT_CUDA="$(lspci -nnk | grep -iA2 NVIDIA | wc -l)"

# translate our cuda major.minor.patch-revision into the cuda meta package name
# eg: 9.1.85-1 becomes 9-1
CUDA_META_VERSION=$(config-get cuda-version | awk 'BEGIN{FS="."; OFS="-"}{print $1, $2}')
GRAPHICS_VERSION=$(config-get graphics-version)

#####################################################################
#
# Basic Functions
#
#####################################################################

function bash::lib::get_ubuntu_codename() {
    lsb_release -a 2>/dev/null | grep Codename | awk '{ print $2 }'
}

UBUNTU_CODENAME="$(bash::lib::get_ubuntu_codename)"

case "${UBUNTU_CODENAME}" in
    "xenial" )
        UBUNTU_VERSION=ubuntu1604
    ;;
    "bionic" )
        UBUNTU_VERSION=ubuntu1804
    ;;
    * )
        juju-log "Your version of Ubuntu is not supported. Exiting"
        exit 1
    ;;
esac


#####################################################################
#
# Handle prerequisite steps
#
#####################################################################

function all::all::prereqs() {
    # Blacklist the nouveau module, which conflicts with nvidia.
    # NB: do not remove nouveau pkgs because of the many x11/openjdk deps. For
    # example, removing libdrm-nouveau* also removes openjdk-8-[jre|jdk].
    #   BAD: apt-get remove -yqq --purge libdrm-nouveau*
    #        apt-get -yqq autoremove
    #   GOOD: unload and blacklist the module
    juju-log "Blacklisting nouveau module"
    modprobe --remove nouveau || \
        juju-log "No nouveau module was found. No modprobe action is needed."
    cat > ${LAYER_BLACKLIST_CONF} <<EOF
blacklist nouveau
blacklist lbm-nouveau
options nouveau modeset=0
alias nouveau off
alias lbm-nouveau off
EOF
    # update the initramfs since we altered our module blacklist
    update-initramfs -u

    juju-log "Installing linux-image-extra"
    # some kernels do not have an -extra subpackage; proceed anyway
    apt-get install -yqq  linux-image-extra-`uname -r` || \
        juju-log "linux-image-extra-`uname -r` not available. Skipping"
}

#####################################################################
#
# Purge nvidia packages per architecture
#
#####################################################################


function xenial::x86_64::purge_nvidia_driver() {
    all:all:purge_nvidia_driver
}

function xenial::ppc64le::purge_nvidia_driver() {
    bash::lib::log info "This task is handled by the cuda installer"
}
function bionic::ppc64le::purge_nvidia_driver() {
    bash::lib::log info "This task is handled by the cuda installer"
}
function bionic::x86_64::purge_nvidia_driver() {
    all:all:purge_nvidia_driver
}


function all:all:install_graphics() {
    juju-log "Installing graphics drivers nvidia-${GRAPHICS_VERSION}"
    add-apt-repository ppa:graphics-drivers/ppa
    apt-get install -yqq --no-install-recommends \
        nvidia-driver-${GRAPHICS_VERSION}
}

function xenial::x86_64::install_cuda() {
    all::all::install_cuda
}
function bionic::x86_64::install_cuda() {
    all::all::install_cuda
}
function xenial::ppc64le::install_cuda() {
    all::all::install_cuda
}
function bionic::ppc64le::install_cuda() {
    all::all::install_cuda
}

function all::all::remove_cuda_config() {
    status-set maintenance "Removing graphics driver"
    apt purge nvidia-* -y

    # update the initramfs since we altered our module blacklist
    update-initramfs -u
}

#####################################################################
#
# Reactive Handlers
#
#####################################################################

@when_not 'cuda.supported'
function check_cuda_support() {
    case "${SUPPORT_CUDA}" in
        "0" )
            juju-log "This instance does not run an nVidia GPU."
        ;;
        * )
            charms.reactive set_state 'cuda.supported'
        ;;
    esac
}

@when 'cuda.supported'
@when_not 'cuda.installed'
function install_cuda() {
    # Return if we're configured to skip installation
    INSTALL=$(config-get install-cuda)
    if [ "${INSTALL}" = "False" ]; then
      juju-log "Skip cuda installation"
      return
    fi

    status-set maintenance "Installing Graphics Driver"
    all:all:install_graphics
    status-set active "Graphics drivers Installed"
    charms.reactive set_state 'cuda.installed'
}

@when 'cuda.installed'
@when 'config.changed.cuda-version'
function config_cuda_version() {
    # Remove config and reinstall if a new cuda-repo version is configured
    if dpkg -l nvidia-driver-* 2>/dev/null | grep -q "${GRAPHICS_VERSION}"; then
        juju-log "nvidia-driver-${GRAPHICS_VERSION} is already installed"
    else
        juju-log "Reinstalling with new grahics driver version"
        all::all::remove_cuda_config
        install_cuda
    fi
}

@when 'cuda.installed'
@when 'config.changed.install-cuda'
function config_cuda_install() {
    # Remove config if user sets install-cuda to false
    INSTALL=$(config-get install-cuda)
    if [ "${INSTALL}" = "False" ]; then
        juju-log "Removing CUDA configuration"
        all::all::remove_cuda_config
        charms.reactive remove_state 'cuda.installed'
    fi
}

reactive_handler_main
